import { IElement, IShape, IGroup, ICanvas } from '../interfaces';
import { ClipCfg, ChangeType, ShapeAttrs, BBox } from '../types';
import Base from './base';
declare abstract class Element extends Base implements IElement {
    /**
     * @protected
     * 图形属性
     * @type {ShapeAttrs}
     */
    attrs: ShapeAttrs;
    constructor(cfg: any);
    getDefaultCfg(): {
        visible: boolean;
        capture: boolean;
    };
    /**
     * @protected
     * 获取默认的属相
     */
    getDefaultAttrs(): {
        matrix: any;
    };
    /**
     * @protected
     * 一些方法调用会引起画布变化
     * @param {ChangeType} changeType 改变的类型
     */
    onCanvasChange(changeType: ChangeType): void;
    /**
     * @protected
     * 初始化属性，有些属性需要加工
     * @param {object} attrs 属性值
     */
    initAttrs(attrs: ShapeAttrs): void;
    /**
     * @protected
     * 初始化动画
     */
    initAnimate(): void;
    isGroup(): boolean;
    getParent(): IGroup;
    getCanvas(): ICanvas;
    attr(...args: any[]): any;
    abstract getBBox(): BBox;
    abstract getCanvasBBox(): BBox;
    isClipped(refX: any, refY: any): boolean;
    /**
     * 内部设置属性值的接口
     * @param {string} name 属性名
     * @param {any} value 属性值
     */
    setAttr(name: string, value: any): void;
    /**
     * @protected
     * 属性值发生改变
     * @param {string} name 属性名
     * @param {any} value 属性值
     * @param {any} originValue 属性值
     */
    onAttrChange(name: string, value: any, originValue: any): void;
    /**
     * 属性更改后需要做的事情
     * @protected
     */
    afterAttrsChange(targetAttrs: any): void;
    show(): this;
    hide(): this;
    toFront(): void;
    toBack(): void;
    remove(destroy?: boolean): void;
    resetMatrix(): void;
    getMatrix(): number[];
    setMatrix(m: number[]): void;
    getTotalMatrix(): any;
    applyMatrix(matrix: number[]): void;
    /**
     * @protected
     * 获取默认的矩阵
     * @returns {number[]|null} 默认的矩阵
     */
    getDefaultMatrix(): any;
    applyToMatrix(v: number[]): number[];
    invertFromMatrix(v: number[]): number[];
    setClip(clipCfg: ClipCfg): any;
    getClip(): IShape;
    clone(): any;
    destroy(): void;
    /**
     * 是否处于动画暂停状态
     * @return {boolean} 是否处于动画暂停状态
     */
    isAnimatePaused(): any;
    /**
     * 执行动画，支持多种函数签名
     * 1. animate(toAttrs: ElementAttrs, duration: number, easing?: string, callback?: () => void, delay?: number)
     * 2. animate(onFrame: OnFrame, duration: number, easing?: string, callback?: () => void, delay?: number)
     * 3. animate(toAttrs: ElementAttrs, cfg: AnimateCfg)
     * 4. animate(onFrame: OnFrame, cfg: AnimateCfg)
     * 各个参数的含义为:
     *   toAttrs  动画最终状态
     *   onFrame  自定义帧动画函数
     *   duration 动画执行时间
     *   easing   动画缓动效果
     *   callback 动画执行后的回调
     *   delay    动画延迟时间
     */
    animate(...args: any[]): void;
    /**
     * 停止动画
     * @param {boolean} toEnd 是否到动画的最终状态
     */
    stopAnimate(toEnd?: boolean): void;
    /**
     * 暂停动画
     */
    pauseAnimate(): this;
    /**
     * 恢复动画
     */
    resumeAnimate(): this;
}
export default Element;
