/**
 * @fileoverview 操作 Group 的一些方法，便于进行单元测试
 * @author dxq613@gmail.com
 */
import { IGroup, IElement } from '../interfaces';
declare const GroupUtil: {
    /**
     * 获取 Group 的第一个元素
     * @param  {IGroup}   group Group 对象
     * @return {IElement} 第一个元素
     */
    getFirst(group: IGroup): IElement;
    /**
     * 获取 Group 的最后一个元素
     * @param  {IGroup}   group Group 对象
     * @return {IElement} 元素
     */
    getLast(group: IGroup): IElement;
    /**
     * 子元素的数量
     * @param  {IGroup} group Group 对象
     * @return {number} 子元素数量
     */
    getCount(group: IGroup): number;
    /**
     * 查找所有匹配的元素
     * @param  {IGroup}          group Group 对象
     * @param  {Function}        fn    匹配函数
     * @return {IElement[]} 元素数组
     */
    findAll(group: IGroup, fn: Function): IElement[];
    /**
     * 根据 ID 查找元素
     * @param {IGroup} group Group 对象
     * @param {string} id 元素 id
     * @return {IElement|null} 元素
     */
    findById(group: IGroup, id: string): IElement;
    /**
     * 查找元素，找到第一个返回
     * @param  {IGroup}   group Group 对象
     * @param  {Function} fn    匹配函数
     * @return {IElement|null} 元素，可以为空
     */
    find(group: IGroup, fn: Function): IElement;
};
export default GroupUtil;
