"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var util_1 = require("./util");
var GroupUtil = {
    /**
     * 获取 Group 的第一个元素
     * @param  {IGroup}   group Group 对象
     * @return {IElement} 第一个元素
     */
    getFirst: function (group) {
        return group.getChildren()[0];
    },
    /**
     * 获取 Group 的最后一个元素
     * @param  {IGroup}   group Group 对象
     * @return {IElement} 元素
     */
    getLast: function (group) {
        var children = group.getChildren();
        return children[children.length - 1];
    },
    /**
     * 子元素的数量
     * @param  {IGroup} group Group 对象
     * @return {number} 子元素数量
     */
    getCount: function (group) {
        return group.getChildren().length;
    },
    /**
     * 查找所有匹配的元素
     * @param  {IGroup}          group Group 对象
     * @param  {Function}        fn    匹配函数
     * @return {IElement[]} 元素数组
     */
    findAll: function (group, fn) {
        var rst = [];
        var children = group.getChildren();
        util_1.each(children, function (element) {
            if (fn(element)) {
                rst.push(element);
            }
            if (element.isGroup()) {
                rst = rst.concat(GroupUtil.findAll(element, fn));
            }
        });
        return rst;
    },
    /**
     * 根据 ID 查找元素
     * @param {IGroup} group Group 对象
     * @param {string} id 元素 id
     * @return {IElement|null} 元素
     */
    findById: function (group, id) {
        return this.find(group, function (element) {
            return element.get('id') === id;
        });
    },
    /**
     * 查找元素，找到第一个返回
     * @param  {IGroup}   group Group 对象
     * @param  {Function} fn    匹配函数
     * @return {IElement|null} 元素，可以为空
     */
    find: function (group, fn) {
        var rst = null;
        var children = group.getChildren();
        util_1.each(children, function (element) {
            if (fn(element)) {
                rst = element;
            }
            else if (element.isGroup()) {
                rst = GroupUtil.find(element, fn);
            }
            if (rst) {
                return false;
            }
        });
        return rst;
    },
};
exports.default = GroupUtil;
//# sourceMappingURL=group.js.map